extern crate chrono;
extern crate colored;
#[macro_use]
extern crate log;
extern crate serde;
#[macro_use]
extern crate serde_derive;
extern crate toml;

mod app;
mod apps;
mod logger;
mod proc;
mod result;

use apps::Apps;
use proc::Proc;
use result::Result;

fn main() {
	if let Err(e) = logger::init() {
		println!("FATAL: Error initializing logger: {:?}", e);
		std::process::exit(1);
	}

	match startup() {
		Ok(p) => {
			info!("shutting down");
			for mut proc in p {
				proc.stop();
			}
		}
		Err(e) => {
			error!("FATAL: {:?}", e);
			std::process::exit(1);
		}
	};
}

fn startup() -> Result<Vec<Proc>> {
	let apps = Apps::load("sup.toml")?;
	let (procs, holds) = apps.start();
	let procs = apps.run(procs, holds)?;

	Ok(procs)
}
